<%@ WebHandler language="C#" Class="HttpNSA.NSAHandler" %>

// --------------------------------------------------------------------------------------------------//
// VERSION: 0.01
// --------------------------------------------------------------------------------------------------//
// The primary class of interest is RequestProcessor.  Please see each of the methods in this 
// class for a desciption of the recommended interface.
//
//   General Design Notes:
//   This script contains the class: NSAHandler and RequestProcessor.
//   These classes is provided solely as an example implementation of a server-side HTTP-filing 
//   interface.  As such, Xerox does not make any representations, extend any warranties of any kind, 
//   either expressed or implied, or assumes any responsibilities whatever with respect to this script. 
//   Please pay particular note to the script comments relating to script exploits. All server-side 
//   scripts present the opportunity for misuse and Xerox shall be in no way responsible for any misuse 
//   of this script.
//   This script is presently written to support Microsoft .NET infrastructure version 1.1.
//
// System package includes
using System;
using System.Web;
using System.Net;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;

//=========================================================//
//	class NSAHandler provides the interfaces 			   //
//	for handling a requested operation.				       //
//=========================================================//
// 
// class hierarchy in extended Schlarr-Mellor notation is:
//   NSAHandler --|- IHttpHandler
//
//   class Description:
//   This class provides the interfaces for DeleteFile, GetFile, PutFile, MakeDir, RemoveDir, 
//   and DeleteDirContents  These interfaces are documented with their declarations.
//
namespace HttpNSA
{

//=========================================================//
//	class NSAHandler provides the interfaces 			   //
//	for handling a requested operation.				       //
//=========================================================//
// 
// class hierarchy in extended Schlarr-Mellor notation is:
//   NSAHandler --|- IHttpHandler
//
//   class Description:
//   This class provides the interfaces for responding the IHttpHandler
//   event ProcessRequest.  This class simply redirects the event to
//   the static method ProcessRequest of the RequestProcessor class.
//
	public class NSAHandler: IHttpHandler 
	{
		// Implementation of IHttpHandler's ProcessRequest method,
		// provides redirection to Request Processor class.
		public void ProcessRequest(HttpContext context) 
		{
			RequestProcessor.ProcessRequest(context);
		}
		// Implementation of IHttpHandler's IsReusable property,
		// always returns true.
		public bool IsReusable 
		{
			get 
			{
				return true;
			}	
		}
	}
	
//=========================================================//
//	class RequestProcessor provides the interfaces 			   //
//	for handling a requested operation.				       //
//=========================================================//
// 
// class hierarchy in extended Schlarr-Mellor notation is:
//   RequestProcessor
//
//   class Description:
//   This class provides the interfaces for DeleteFile, GetFile, PutFile, MakeDir, RemoveDir, 
//   and DeleteDirContents  These interfaces are documented with their declarations.
//
	public class RequestProcessor
	{
		// private string class attributes which we'll use as constants to make it easier to synchronize their
		// data with the device code easier.  These attributes represent the "name" of the form 
		// variable in the request stream from the device.
		private const string att_constFormVarNameTheOperation = "theOperation";
		private const string att_constFormVarNameDestinationName = "destName";
		private const string att_constFormVarNameDestinationDirectory = "destDir";
		private const string att_constFormVarNameDeleteDirectory = "delDir";
		// Max acceptable path
		private const int att_constMaxPath = 248;
		// Return codes in the event of an error
		private const string att_constError = "XRXERROR";
		private const string att_constErrorBadName = "XRXBADNAME";
		private const string att_constErrorNotFound = "XRXNOTFOUND";
		private const string att_constErrorDirExists = "XRXDIREXISTS";

		public const string VBLF = "\n";
		public const string DIR_SEPARATOR = "/";

	// Method: RequestProcessor::ProcessRequest
	// This is the callable method of this class.  It performs the actual work of 
	// responding to the form data in the request stream from the device.
	// Expected Form Variables submitted from a device:
	//   theOperation: the name of the operation to perform
	// Expected Behavior:
	//   The ProcessRequest method will call a specific method depending on the operation
	//   requested in the form variable "theOperation".
	// Expected Output:
	//   If successful, nothing.
	//   XRXERROR if "theOperation" isn't supported.
	// Script Exploit Opportunities:
	//   As this function will delegate any real activity to other methods, its primary
	//   responsibility is to only call the other methods on explicit cases of "the operation"
	//   as specified in the incoming form variables.
	// Not Supported:
	//   At present, it is understood that retrieving the script version is not support.
	//   As such, it is not supported by this script.
		static public void ProcessRequest(HttpContext arg_context)
		{
			// This is a funny case, we can't actually write anything out since we don't have
			// a context, but we can't proceed either so we'll just return...
			if(null == arg_context)
				return;

			try
			{
				// Get our forms data
				string strOperation = arg_context.Request.Form[att_constFormVarNameTheOperation];
#if OUTPUT
				arg_context.Response.Write("<b>The operation = " + strOperation + "<br></b>");
#endif
				// Decide what to do based on the forms data
				switch(strOperation)
				{
					case "DeleteFile":
						RequestProcessor.DeleteFile(arg_context);
						break;
					case "MakeDir":
						RequestProcessor.MakeDirectory(arg_context);
						break;
					case "RemoveDir":
						RequestProcessor.RemoveDirectory(arg_context);
						break;
					case "DeleteDirContents":
						RequestProcessor.DeleteDirectoryContents(arg_context);
						break;
					case "GetFile":
						RequestProcessor.GetFile(arg_context);
						break;
					case "PutFile":
						RequestProcessor.SaveFiles(arg_context);
						break;
					case "ListDir":
						RequestProcessor.ListDirectory(arg_context);
						break;
					default:
						// Do nothing except return an error code if the operation isn't one of the above cases.  
						// This narrows the scope of exploit attempts that can be made using the operation as the 
						// entry point.
						arg_context.Response.Write(att_constError);
						break;
				}
			}
			catch
			{
				// Respond with an error if we hit an exception
				arg_context.Response.Write(att_constError);
			}
		}	

	// Method: RequestProcessor::DeleteFile
	// This private method will delete a specific file on the server.
	// Expected Form Variables submitted from a device:
	//   destName: the name of the file to delete, without path information
	//   destDir: the fully qualified file system path of the file to delete,
	//     a slash is optional
	// Expected Behavior:
	//   The delete method will delete the file specified in the form variables
	//   of the request stream.  
	// Expected Output:
	//   If successful, nothing.
	//   XRXBADNAME if the requested file isn't of the correct type or the name is empty.
	//   XRXNOTFOUND if the requested file isn't found. 
	//   XRXERROR the file cannot be deleted.
	// Script Exploit Opportunities:
	//   The delete file operation could be used to delete any file on the system provided
	//   that the running web process has sufficient rights to delete the file.  Since this
	//   places the responsibility of insuring that the web process does not have rights to
	//   delete files it shouldn't be onto the server configuration and that we cannot assume 
	//   that only our files need to be deleted on any given web server, we're going to limit 
	//   the extensions (type) of files that this function will be able to delete.  This way, 
	//   the script cannot be used to delete any file which the web process has rights to delete.
		static private void DeleteFile(HttpContext arg_context)
		{
			// Get the name of the file information from the form variables in the request stream.
			string strDirectoryName = arg_context.Request.Form[att_constFormVarNameDestinationDirectory];
			string strFileName = arg_context.Request.Form[att_constFormVarNameDestinationName];
#if OUTPUT
			arg_context.Response.Write("<b>Directory: " + strDirectoryName + "<br></b>");
			arg_context.Response.Write("<b>File to delete: " + strFileName + "<br></b>");
#endif
			// Verify the file name
			string strTarget = RequestProcessor.BuildFullyQualifiedPath(strDirectoryName, strFileName);
			if((!strTarget.Equals(string.Empty)) && (File.Exists(strTarget)) && RequestProcessor.CheckFileExtension(strTarget))
			{
				// Try and delete it
				try
				{
					File.Delete(strTarget);
				}
				catch (Exception e)
				{
#if OUTPUT
					arg_context.Response.Write("<b>File delete exception: " + e.Message + "<br></b>");
#endif
					arg_context.Response.Write(RequestProcessor.att_constErrorBadName);
				}
			}
			else
			{
				arg_context.Response.Write(RequestProcessor.att_constErrorBadName);
			}
		}
	// Method: RequestProcessor::MakeDirectory
	// This private method will create a directory on the server.
	// Expected Form Variables submitted from a device:
	//   destDir: the fully qualified file system path of the directory to create
	// Expected Behavior:
	//   The make directory method will create the directory specified in the form variables
	//   of the request stream.  
	// Expected Output:
	//   If successful, nothing.
	//   XRXBADNAME if the name is empty.
	//   XRXDIREXISTS if the directory already exists.
	//   XRXERROR if the directory cannot be created.
	// Script Exploit Opportunities:
	//   This function could be used to create enough directories to exceed the file system's
	//   limits.  This script does not prevent this.
		static private void MakeDirectory(HttpContext arg_context)
		{
			// Get the name of the directory from the form variables in the request stream.
			string strDirectoryName = arg_context.Request.Form[att_constFormVarNameDestinationDirectory];
#if OUTPUT
			arg_context.Response.Write("<b>Creating directory: " + strDirectoryName + "<br></b>");
#endif
			// Make sure we have a meaningful directory name to create.
			if((null != strDirectoryName) && (!strDirectoryName.Equals(string.Empty))
				&& (RequestProcessor.att_constMaxPath >= strDirectoryName.Length))
			{
				// Try and create it
				try
				{
					Directory.CreateDirectory(strDirectoryName);
				}
				catch (Exception e)
				{
#if OUTPUT
					arg_context.Response.Write("<b>Directory create exception: " + e.Message + "<br></b>");
#endif
					arg_context.Response.Write(RequestProcessor.att_constErrorBadName);
				}
			}
			else
			{
				arg_context.Response.Write(RequestProcessor.att_constErrorBadName);
			}
		}
	// Method: RequestProcessor::RemoveDirectory
	// This private method will remove a directory on the server.  The directory must end in
	// xsm, lck.  This minimizes the script from being exploited to delete unprotected areas of
	// the file system.
	// Expected Form Variables submitted from a device:
	//   destDir: the fully qualified file system path of the directory to remove.
	// Expected Behavior:
	//   The remove directory method will remove the directory specified in the form variables
	//   of the request stream.  
	// Expected Output:
	//   If successful, nothing
	//   XRXBADNAME if the requested directory isn't of the correct type or the name is empty.
	//   XRXNOTFOUND if the directory does not exist.
	//   XRXERROR if the directory cannot be deleted.
	// Script Exploit Opportunities:
	//   The remove directory operation could be used to remove any folder on the system provided
	//   that the running web process has sufficient rights to remove the folder.  Since this
	//   places the responsibility of insuring that the web process does not have rights to
	//   remove folders it shouldn//t be onto the server configuration and that we cannot assume 
	//   that only our folders need to be removed on any given web server, we're going to limit 
	//   the extensions (type) of folders that this function will be able to delete.  This way, 
	//   the script cannot be used to remove any folder which the web process has rights to remove.
		static private void RemoveDirectory(HttpContext arg_context)
		{
			// Get the name of the directory from the form variables in the request stream.
			string strDirectoryName = arg_context.Request.Form[att_constFormVarNameDestinationDirectory];
			RequestProcessor.RemoveDirectory(arg_context, strDirectoryName);
		}
		// Helper function to use common code between interface's RemoveDirectory and DelDirectoryContents
		static private void RemoveDirectory(HttpContext arg_context, string arg_str)
		{
			// Get the name of the directory to list from the form variables in the request stream.
			string strDirectoryName = arg_str;
#if OUTPUT
			arg_context.Response.Write("<b>Removing directory: " + strDirectoryName + "<br></b>");
#endif
			// Make sure we have a valid directory name.
			if(RequestProcessor.CheckDirectoryExtension(strDirectoryName))
			{
				try
				{
					Directory.Delete(strDirectoryName, true);
				}
				catch (Exception e)
				{
#if OUTPUT
					arg_context.Response.Write("<b>Directory delete exception: " + e.Message + "<br></b>");
#endif
					arg_context.Response.Write(RequestProcessor.att_constErrorBadName);
				}
			}
			else
			{
				arg_context.Response.Write(RequestProcessor.att_constErrorBadName);
			}
		}
	// Method: RequestProcessor::DeleteDirectoryContents
	// This private method will remove a directory and all of its subdirectories and files 
	// on the server.  The directory must end in ".lck".  This minimizes the script from 
	// being exploited to delete unprotected areas of the file system.
	// Expected Form Variables submitted from a device:
	//   deleteDir: Zero to remove the directory also, One to keep the directory.
	//   destDir: the fully qualified file system path of the directory to remove
	// Expected Behavior:
	//   The delete directory contents method will delete the directory and all of its
	//   files and subdirectories specified in the form variables.
	//   of the request stream.  
	// Expected Output:
	//   If successful, nothing
	//   XRXBADNAME if the requested directory isn't of the correct type or the name is empty.
	//   XRXNOTFOUND if the directory does not exist.
	//   XRXERROR if the directory (or some of its files) cannot be deleted.
	// Not Supported:
	//   At present, the form variable "isRecursive" is not supported. The device team//s test 
	//   HTML form contains this variable, but it is not used by the existing server-side script. 
	//   As such, it is not supported in this script.
	// Script Exploit Opportunities:
	//   The delete directory contents operation could be used to remove any folder and contained
	//   subfolders and files on the system provided that the running web process has sufficient 
	//   rights to remove the folder.  Since this places the responsibility of insuring that the 
	//   web process does not have rights to remove folders and delete files onto the server 
	//   configuration and that we cannot assume that only our folders need to be removed on any 
	//   given web server, we're going to limit the extensions (type) of folders that this function 
	//   will be able to delete.  This way, the script cannot be used to remove any folder which the 
	//   web process has rights to remove.
		static private void DeleteDirectoryContents(HttpContext arg_context)
		{
			// Get the name of the directory from the form variables in the request stream.
			string strDirectoryName = arg_context.Request.Form[att_constFormVarNameDestinationDirectory];
			string strDeleteDirectory = arg_context.Request.Form[att_constFormVarNameDeleteDirectory];
#if OUTPUT
			arg_context.Response.Write("<b>Deleting directory contents: " + strDirectoryName + "<br></b>");
			arg_context.Response.Write("<b>Also delete directory: " + (strDeleteDirectory.Equals("1")?"yes":"no") + "<br></b>");
#endif
			// If we're deleting the whole directory, just call RemoveDirectory
			if(strDeleteDirectory.Equals("1"))
			{
				RequestProcessor.RemoveDirectory(arg_context);
			}
			// Otherwise, check our extension and...
			else if(RequestProcessor.CheckDirectoryExtension(strDirectoryName))
			{
				DirectoryInfo di = new DirectoryInfo(strDirectoryName);
				if(di.Exists)
				{
					// Wipe out all of our subdirectories
					DirectoryInfo[] dirs = di.GetDirectories();
					foreach (DirectoryInfo diNext in dirs)
					{
						try
						{
							Directory.Delete(diNext.FullName, true);
						}
						// If something went wrong, just continue
						catch{}
					}
					// Wipe out all of our files
					FileInfo[] fi = di.GetFiles();
					foreach (FileInfo fiTemp in fi)
					{
						try
						{
							File.Delete(fiTemp.FullName);
						}
						// If something went wrong, just continue
						catch{}
					}
				}
				else
					arg_context.Response.Write(RequestProcessor.att_constErrorNotFound);
			}
			else
			{
				arg_context.Response.Write(RequestProcessor.att_constErrorBadName);
			}
		}
	// Method: RequestProcessor::GetFile
	// This private method will return the contents of the specified file as text
	// the repsonse stream.  To prevent this function being exploited and used to
	// list the contents of potentially any text-based file accessible to the server,
	// this function only returns the contents of files with the xst, dat extensions.
	// Expected Form Variables submitted from a device:
	//   destDir: the fully qualified file system path of the file to return
	//   destName: the name of the file to return
	// Expected Behavior:
	//   The get file method will return the contents the file specified 
	//   in the form variables of the request stream.  
	// Expected Output:
	//   If successful, return the contents of the file in the response stream.
	//   XRXBADNAME if the requested file isn't of the correct type or the name is empty.
	//   XRXNOTFOUND if the requested file isn't found.
	//   XRXERROR the file cannot be read.
	// Script Exploit Opportunities:
	//   The get file operation could be used to retrieve the contents any text file on the 
	//   system provided that the running web process has sufficient rights to access the file.  
	//   Since this places the responsibility of insuring that the web process does not have 
	//   rights to read files which it shouldn't be able to onto the server configuration and 
	//   that we cannot assume that only our files should be readable on any given web server, 
	//   we're going to limit the extensions (type) of files that this function will be able to
	//   read. This way, the script cannot be used to retrieve the contents of any file which 
	//   the web process has rights to access.
	//   Another attacker could use this function to launch a denial of service attack if the
	//   script were to use the ReadAll function to retrieve the contents of the requested file.
	//   See below for additional details on why not use ReadAll.
		static private void GetFile(HttpContext arg_context)
		{
			// Get the file info from the form variables in the request stream.
			string strDirectoryName = arg_context.Request.Form[att_constFormVarNameDestinationDirectory];
			string strFileName = arg_context.Request.Form[att_constFormVarNameDestinationName];
#if OUTPUT
			arg_context.Response.Write("<b>Directory: " + strDirectoryName + "<br></b>");
			arg_context.Response.Write("<b>File to get: " + strFileName + "<br></b>");
#endif
			// Validate the file's name
			string strTarget = RequestProcessor.BuildFullyQualifiedPath(strDirectoryName, strFileName);
			if((!strTarget.Equals(string.Empty)) && (File.Exists(strTarget) && RequestProcessor.CheckFileExtension(strFileName)))
			{
				// Try and read it
				try
				{
					string line = string.Empty;
					StreamReader fs = new StreamReader(strTarget);
					while (null != (line = fs.ReadLine())) 
					{
						// write out each "chunk"
						arg_context.Response.Write(line + RequestProcessor.VBLF);
					}
					fs.Close();
				}
				catch (Exception e)
				{
#if OUTPUT
					arg_context.Response.Write("<b>File get exception: " + e.Message + "<br></b>");
#endif
					arg_context.Response.Write(RequestProcessor.att_constErrorBadName);
				}
			}
			else
			{
				arg_context.Response.Write(RequestProcessor.att_constErrorBadName);
			}
		}
	// Method: RequestProcessor::SaveFiles
	// This private method will save all of the files post as multi-part form data into 
	// the directory specified in the form variables of the request stream.
	// Expected Form Variables submitted from a device:
	//   destName: the name of the file to delete, without path information.
	//   destDir: the fully qualified file system path of the directory to list
	// Expected Behavior:
	//   The save file method will save the files found in the request to the directory
	//   specified in the request stream.
	//   Note: if accessing this page annonymously, the internet guest account must have
	//   write permission to the path.
	// Expected Output:
	//   If successful, nothing.
	//   XRXBADNAME if the file name is empty.
	//   XRXERROR if the file cannot be written.
	// Script Exploit Opportunities:
	//   The save files operation could be used to upload malicious scripts to the server.
	//   Once uploaded, if the server has execute permission of these scripts, a user can
	//   effectively, remotely invoke the malicious code. The easiest solution to prevent this
	//   would be to remove execute permissions to the files this function writes.  This however
	//   does not apply to Windows-based file systems.  The other alternative would be limit
	//   the types of files this function uploads.  However, doing so would greatly limit this
	//   scripts ability to handle new uploads.  As lack of extensibility is greater than the
	//   risk of the use of this function to upload malicious scripts, this function does not
	//   prevent the upload of potentially malicious scripts.
		static private void SaveFiles(HttpContext arg_context)
		{
			// Get the file info from the form variables in the request stream.
			string strDirectoryName = arg_context.Request.Form[att_constFormVarNameDestinationDirectory];
			string strFileName = arg_context.Request.Form[att_constFormVarNameDestinationName];
#if OUTPUT
			arg_context.Response.Write("<b>Directory: " + strDirectoryName + "<br></b>");
			arg_context.Response.Write("<b>File to save: " + strFileName + "<br></b>");
#endif
			// Setup the file info
			string strTarget = RequestProcessor.BuildFullyQualifiedPath(strDirectoryName, strFileName);
			if((!strTarget.Equals(string.Empty)))
			{
				try
				{
					HttpFileCollection files = arg_context.Request.Files;
					if(0 < files.Count)
					{
						// Only process one file, telling it to save itself to the file info
						HttpPostedFile file = files[0];
						file.SaveAs(strTarget);
					}
					else
					{
						arg_context.Response.Write(RequestProcessor.att_constError);
					}
				}
				catch (Exception e)
				{
#if OUTPUT
					arg_context.Response.Write("<b>File save exception: " + e.Message + "<br></b>");
#endif
					arg_context.Response.Write(RequestProcessor.att_constError);
				}
			}
			else
			{
				arg_context.Response.Write(RequestProcessor.att_constErrorBadName);		
			}
		}
	// Method: RequestProcessor::ListDirectory
	// This private method will list all of the files in a directory specified in 
	// the form variables in the request stream.  It will also lists any sub directories,
	// but not the files in the sub directories or their sub directories.
	// Expected Form Variables submitted from a device:
	//   destDir: the fully qualified file system path of the directory to list
	// Expected Behavior:
	//   The list directory method will enumerate the contents of the directory specified 
	//   in the form variables of the request stream.  
	// Expected Output:
	//   Nothing if no contents are found.  Otherwise it returns a line-feed (ASCII 10)
	//   delimited list of files and subdirectories.  Subdirectories are noted by appending
	//   the forward slash ("/") character to the end of the subdirectory's name.
	//   Otherwise: XRXBADNAME if the requested directory name is empty.
	//   XRXNOTFOUND if the directory does not exist.
	//   XRXERROR if the directory cannot be read.
	// Script Exploit Opportunities:
	//   This function could be used to determine the name of the files within a directory
	//   and use those names for malicious purposes like deleting files.  As this script
	//   limits activities like deleting files and dirctories, the only risk is that another
	//   script is used to perform the malicious activities. This script does not prevent that.
		static private void ListDirectory(HttpContext arg_context)
		{
			// Get the name of the directory to list from the form variables in the request stream.
			string strDirectoryName = arg_context.Request.Form[att_constFormVarNameDestinationDirectory];
#if OUTPUT
			arg_context.Response.Write("<b>Listing directory: " + strDirectoryName + "<br></b>");
#endif
			// Make sure we have a meaningful directory name to list.
			if((null != strDirectoryName) && (!strDirectoryName.Equals(string.Empty))
				&& (RequestProcessor.att_constMaxPath >= strDirectoryName.Length))
			{
				// Create a reference to the directory.
				DirectoryInfo di = new DirectoryInfo(strDirectoryName);
				if(di.Exists)
				{
					// Write out our directory members
					DirectoryInfo[] dirs = di.GetDirectories();
					foreach (DirectoryInfo diNext in dirs)
						arg_context.Response.Write(diNext.Name + RequestProcessor.DIR_SEPARATOR + RequestProcessor.VBLF);
					// Write out our file members
					FileInfo[] fi = di.GetFiles();
					foreach (FileInfo fiTemp in fi)
						arg_context.Response.Write(fiTemp.Name + RequestProcessor.VBLF);
				}
				else
					arg_context.Response.Write(RequestProcessor.att_constErrorNotFound);
			}
		}
	// Method: RequestProcessor::CheckFileExtension
	// Internal Helper Only
	// To help with processing time of the script, the construction of the file name extensions
	// to check and the check itself have been encapsulated into this function.  This way, the
	// work is only done when the function is called.  Please note that the function is not
	// presently called more than once per any form post.  That enables the payoff of this 
	// strategy.
		static private bool CheckFileExtension(string arg_strPath)
		{
			// Create a hash table to enable quick matching of valid file extensions
			Hashtable htValidExtensions = new Hashtable();
			htValidExtensions.Add(".xst", null);
			htValidExtensions.Add(".dat", null);
			htValidExtensions.Add(".txt", null);
			// Call the internal helper to execute the search
			return RequestProcessor.CheckExtension(htValidExtensions, arg_strPath);
		}
	// Method: RequestProcessor::CheckDirectoryExtension
	// Internal Helper Only
	// To help with processing time of the script, the construction of the directory name extensions
	// to check and the check itself have been encapsulated into this function.  This way, the
	// work is only done when the function is called.  Please note that the function is not
	// presently called more than once per any form post.  That enables the payoff of this 
	// strategy.
		static private bool CheckDirectoryExtension(string arg_strPath)
		{
			// Create a hash table to enable quick matching of valid directory extensions
			Hashtable htValidExtensions = new Hashtable();
			htValidExtensions.Add(".lck", null);
			htValidExtensions.Add(".xsm", null);
			// Call the internal helper to execute the search
			return RequestProcessor.CheckExtension(htValidExtensions, arg_strPath);
		}
	// Method: RequestProcessor::CheckExtension
	// Internal Helper Only
	// Internal method for checking extension based on list
		static private bool CheckExtension(Hashtable arg_ht, string arg_strPath)
		{
			// Validate our incoming arguments
			if((null == arg_strPath) || (null == arg_ht)
				|| (0 == arg_strPath.Length) || (-1 == arg_strPath.IndexOf(".")))
				return false;
			// Extract our extension
			string strExtension = Path.GetExtension(arg_strPath).ToLower();
			// Execute the search
			return (null == strExtension) ? false : arg_ht.ContainsKey(strExtension);
		}
	// Method: RequestProcessor::BuildFullyQualifiedPath
	// Internal Helper Only
	// This function ensures that there is a slash on the end of the path before appending
	// the file name onto it.
		static private string BuildFullyQualifiedPath(string arg_dirName, string arg_fileName)
		{
			// Setup our return variable
			string strReturn = string.Empty;
			// Validate our incoming arguments
			if((null != arg_dirName) && (null != arg_fileName)
				&& (RequestProcessor.att_constMaxPath >= arg_dirName.Length) && (RequestProcessor.att_constMaxPath >= arg_fileName.Length)
				&& (!arg_dirName.Equals(string.Empty)) && (!arg_fileName.Equals(string.Empty)))
			{
				// Combine our arguments
				strReturn = Path.Combine(arg_dirName, arg_fileName);
			}
			// Return empty or the combined value
			return strReturn;
		}
	}
}


