<%
' --------------------------------------------------------------------------------------------------'
' VERSION: 0.08
' --------------------------------------------------------------------------------------------------'
' Start of the class library section of this code
' The actual running part of this script may be found at the very bottom of this file.
' 
' The primary class of interest is vbsRequestExecutor.  Please see each of the methods in this 
' class for a desciption of the recommended interface.
'
'   General Design Notes:
'   This script contains four classes: vbsRequestExecutor, vbsUpload, vbsCollection, and vbsFile.
'   vbsRequestExecutor and is provided solely as an example implementation of a server-side 
'   HTTP-filing interface.  As such, Xerox does not make any representations, extend any warranties 
'   of any kind, either expressed or implied, or assumes any responsibilities whatever with respect 
'   to this script. Please pay particular note to the script comments relating to script exploits. 
'   All server-side scripts present the opportunity for misuse and Xerox shall be in no way 
'   responsible for any misuse of this script.
'   This script does not use global or public constants.  This is to prevent the exploitation
'   of the constants in the event that the script is included by a malicious script.
'   Whenever a constant is used in a conditional expression, this script places the constant
'   on the left-hand side of the expression.  This is to provided consistency with scripts
'   written in languages which support inadvertent assignment instead of evaluation.  For
'   example, "if(a = 100)" can evaulate very differently than "if(100 == a)".  In the latter
'   case, if the expression was "if(100 = a)" and the languauge supports assignment in
'   conditionals, this will result in an error instead of returning "true" when the assignment
'   is complete.
'
'   Please note that this code make use of the ASP Err class.  Please refer to the MSDN documentation
'   Visual Basic Scripting Edition -> On Error Statement for information on the use of error handling
'   in ASP and the use of the Err class.
'
'========================================================='
'	Class vbsRequestExecutor provides the interfaces 	  '
'	for handling a requested operation.				  '
'========================================================='
' 
' Class hierarchy in extended Schlarr-Mellor notation is:
'   vbsRequestExecutor
'   vbsRequestExecutor::Run -> (vbsUload -> vbsCollection ->> vbsFile)
'
'   Class Description:
'   This class provides the interfaces for DeleteFile, GetFile, PutFile, MakeDir, RemoveDir, 
'   and DeleteDirContents  These interfaces are documented with their declarations.
'
Class vbsRequestExecutor
	' Private class attributes which we'll use as constants to make it easier to synchronize their
	' data with the device code easier.  These attributes represent the "name" of the form 
	' variable in the request stream from the device.
	Private att_constFormVarNameTheOperation
	Private att_constFormVarNameDestinationName
	Private att_constFormVarNameDestinationDirectory
	Private att_constFormVarNameDeleteDirectory
	' Max acceptable fully qualified path
	Private att_constMaxPath
	' Return codes in the event of an error
	Private att_constError
	Private att_constErrorBadName
	Private att_constErrorNotFound
	Private att_constErrorDirExists

	' Class initializer.  Make sure we've set all of our attributes to the initial values.
	Private Sub Class_Initialize()
		' Remember to make sure that these are in synch with the device code.
		att_constFormVarNameTheOperation = "theOperation"
		att_constFormVarNameDestinationName = "destName"
		att_constFormVarNameDestinationDirectory = "destDir"
		att_constFormVarNameDeleteDirectory = "delDir"
		' Max path has been arbitrarily set to 64k, basically to prevent someone from
		' sending us a zillion bytes and then crashing us because we try to use them.
		att_constMaxPath = 65536
		' These values will be returned in the response stream in the event of an error.
		att_constError = "XRXERROR"
		att_constErrorBadName = "XRXBADNAME"
		att_constErrorNotFound = "XRXNOTFOUND"
		att_constErrorDirExists = "XRXDIREXISTS"
	End Sub

	' Method: vbsRequestExecutor::Run
	' This is the callable method of this class.  It performs the actual work of 
	' responding to the form data in the request stream from the device.
	' Expected Form Variables submitted from a device:
	'   theOperation: the name of the operation to perform
	' Expected Behavior:
	'   The Run method will call a specific method depending on the operation
	'   requested in the form variable "theOperation".
	' Expected Return:
	'   If successful, nothing.
	'   XRXERROR if "theOperation" isn't supported.
	' Script Exploit Opportunities:
	'   As this function will delegate any real activity to other methods, it's primary
	'   responsibility is to only call the other methods on explicit cases of "the operation"
	'   as specified in the incoming form variables.
	' Not Supported:
	'   At present, it is understood that retrieving the script version is not support.
	'   As such, it is not supported by this script.
	Public Sub Run()
		' Our local variables
		Dim objUpload, strTheOperation
		' If there are bytes in the request stream...
		If 0 < Request.TotalBytes Then
			' Get us our workhorse object which will extract all of the relevant data
			' out of the request stream during its construction.
			Set objUpload = New vbsUpload

			' Get the thing that we want to do from the form variables in the request stream
			strTheOperation = objUpload.Form.Item(att_constFormVarNameTheOperation)
			' and then call the appropriate method to do it.
			If "DeleteFile" = strTheOperation Then
				DeleteFile(objUpload)
			ElseIf "MakeDir" = strTheOperation Then
				MakeDirectory(objUpload)
			ElseIf "RemoveDir" = strTheOperation Then
				RemoveDirectory(objUpload)
			ElseIf "DeleteDirContents" = strTheOperation Then
				DeleteDirectoryContents(objUpload)
			ElseIf "GetFile" = strTheOperation Then
				GetFile(objUpload)
			ElseIf "PutFile" = strTheOperation Then
				SaveFiles(objUpload)
			ElseIf "ListDir" = strTheOperation Then
				ListDirectory(objUpload)
			Else
				' Send an error code back
				Response.Write att_constError
			End if
			' Do nothing if the operation isn't one of the above cases.  This narrows the scope
			' of exploit attempts that can be made using the operation as the entry point.

			' Make sure we allow the upload worker object to be garbage collected.
			Set objUpload = Nothing
		Else
			' Send an error code back
			Response.Write att_constError
		End if
		' vbsRequestExecutor::Run is done.
	End Sub

	' Method: vbsRequestExecutor::DeleteFile
	' This private method will delete a specific file on the server.
	' Expected Form Variables submitted from a device:
	'   destName: the name of the file to delete, without path information
	'   destDir: the fully qualified file system path of the file to delete,
	'     a slash is optional
	' Expected Behavior:
	'   The delete method will delete the file specified in the form variables
	'   of the request stream.  
	' Expected Return:
	'   If successful, nothing.
	'   XRXBADNAME if the requested file isn't of the correct type or the name is empty.
	'   XRXNOTFOUND if the requested file isn't found. 
	'   XRXERROR the file cannot be deleted.
	' Script Exploit Opportunities:
	'   The delete file operation could be used to delete any file on the system provided
	'   that the running web process has sufficient rights to delete the file.  Since this
	'   places the responsibility of insuring that the web process does not have rights to
	'   delete files it shouldn't be onto the server configuration and that we cannot assume 
	'   that only our files need to be deleted on any given web server, we're going to limit 
	'   the extensions (type) of files that this function will be able to delete.  This way, 
	'   the script cannot be used to delete any file which the web process has rights to delete.
	Private Sub DeleteFile(arg_objUpload)
		' Our local variables.
		Dim objFSO, strFileName, strPathName, strTargetName, strReturn
		' Initialize our return state
		strReturn = ""
		' Get the path of the file to delete from the form variables in the request stream.
		strPathName = arg_objUpload.Form.Item(att_constFormVarNameDestinationDirectory)
		' Get the name of the file to delete from the form variables in the request stream.
		strTargetName = arg_objUpload.Form.Item(att_constFormVarNameDestinationName)
		' Make sure that the file is one of the types we allow to be deleted
		If CheckFileExtension(strTargetName) Then
			' Build the fully qualified target path using the directory and the file name.
			strFileName = BuildFullyQualifiedPath(strPathName, strTargetName)
			' Make sure we have a meaningful file name to delete.
			If ("" <> strFileName) Then
				' Setup an error handler just in case something goes wrong
				On Error Resume Next
				' Get us somebody to interact with the file system.
				Set objFSO = Server.CreateObject("Scripting.FileSystemObject")
				' Make sure that we actually have something to delete.
				If objFSO.FileExists(strFileName) Then
					' Delete the file. NOTE: There's no return code to check for the delete file
					' function.  The caller will have to check the return status from the web
					' server.
					objFSO.DeleteFile(strFileName)
					' Only flag the error if we have one
					If (0 <> Err.Number) Then
						' Flag the error
						strReturn = att_constError
						' Clear the error so that the rest of the code can proceed
						Err.Clear
					End If
				Else
					' The file wasn't found
					strReturn = att_constErrorNotFound
				End If
				' Make sure we allow the file system object to be garbage collected.
				Set objFSO = Nothing
			Else
				' The file name is empty, that's bad
				strReturn = att_constErrorBadName
			End If
		Else
			' The requested type of file isn't one we're supposed to be deleting, that's bad
			strReturn = att_constErrorBadName
		End If
		' See if we should send an error back
		If ("" <> strReturn) Then
			Response.Write strReturn
		End If
		' vbsRequestExecutor::DeleteFile is done.
	End Sub

	' Method: vbsRequestExecutor::GetFile
	' This private method will return the contents of the specified file as text
	' the repsonse stream.  To prevent this function being exploited and used to
	' list the contents of potentially any text-based file accessible to the server,
	' this function only returns the contents of files with the xst, dat extensions.
	' Expected Form Variables submitted from a device:
	'   destDir: the fully qualified file system path of the file to return
	'   destName: the name of the file to return
	' Expected Behavior:
	'   The get file method will return the contents the file specified 
	'   in the form variables of the request stream.  
	' Expected Return:
	'   If successful, return the contents of the file in the response stream.
	'   XRXBADNAME if the requested file isn't of the correct type or the name is empty.
	'   XRXNOTFOUND if the requested file isn't found.
	'   XRXERROR the file cannot be read.
	' Script Exploit Opportunities:
	'   The get file operation could be used to retrieve the contents any text file on the 
	'   system provided that the running web process has sufficient rights to access the file.  
	'   Since this places the responsibility of insuring that the web process does not have 
	'   rights to read files which it shouldn't be able to onto the server configuration and 
	'   that we cannot assume that only our files should be readable on any given web server, 
	'   we're going to limit the extensions (type) of files that this function will be able to
	'   read. This way, the script cannot be used to retrieve the contents of any file which 
	'   the web process has rights to access.
	'   Another attacker could use this function to launch a denial of service attack if the
	'   script were to use the ReadAll function to retrieve the contents of the requested file.
	'   See below for additional details on why not use ReadAll.
	Private Sub GetFile(arg_objUpload)
		' Our local variables.
		Dim objFSO, strFileName, strPathName, strTargetName, objFile, strReturn
		' Initialize our return state
		strReturn = ""
		' Get the path of the file to retrieve from the form variables in the request stream.
		strPathName = arg_objUpload.Form.Item(att_constFormVarNameDestinationDirectory)
		' Get the name of the file to retrieve from the form variables in the request stream.
		strTargetName = arg_objUpload.Form.Item(att_constFormVarNameDestinationName)
		' Make sure that the file is one of the types we allow to be retrieved
		If CheckFileExtension(strTargetName) Then
			' Build the fully qualified target path using the directory and the file name.
			strFileName = BuildFullyQualifiedPath(strPathName, strTargetName)
			' Make sure we have a meaningful file name to retrieve.
			If ("" <> strFileName) Then
				' Setup an error handler just in case something goes wrong
				On Error Resume Next
				' Get us somebody to interact with the file system.
				Set objFSO = Server.CreateObject("Scripting.FileSystemObject")
				' Make sure that we actually have something to delete.
				If objFSO.FileExists(strFileName) Then
					' Open the file
					Set objFile = objFSO.OpenTextFile(strFileName) 
					' Only flag the error if we have one
					If (0 <> Err.Number) Then
						' Flag the error
						strReturn = att_constError
						' Clear the error so that the rest of the code can proceed
						Err.Clear
						' Make sure our object is null
						objFile = Null
					End If
					' Make sure the file is open
					' Normally the following error checking would be a good idea, 
					' however these cases are always correct:
					'If((Null = objFile) Or (Empty = objFile) Or (objFile.AtEndOfStream)) Then
						' This is bad, make note of the error
					'	strReturn = att_constError
					'Else
						' While there is something to read
						Do While Not objFile.AtEndOfStream
							' Read it, making sure to add any eaten newlines back in.
							' The script is using ReadLine specifically because Read
							' is a pain to use and ReadAll can be used as a sort of 
							' denial of service (DOS) attack.  The DOS attack could
							' be accomplished by uploading a large enough file which
							' the script is allowed to read (like an .xst) and then
							' sending lots of request for the script to read it.  With
							' ReadAll, the entire file is pulled into memory before
							' the function returns, instead of a part at a time.
							Response.Write objFile.ReadLine & vblf
						Loop
						' Make sure we close the file reader.
						objFile.Close()
					'End If
				Else
					' The file wasn't found
					strReturn = att_constErrorNotFound
				End If
				' Make sure we allow the file system object to be garbage collected.
				Set objFSO = Nothing
			Else
				' The file name is empty, that's bad
				strReturn = att_constErrorBadName
			End If
		Else
			' The requested type of file isn't one we're supposed to be deleting, that's bad
			strReturn = att_constErrorBadName
		End If
		' See if we should send an error back
		If ("" <> strReturn) Then
			Response.Write strReturn
		End If
		' vbsRequestExecutor::GetFile is done.
	End Sub

	' Method: vbsRequestExecutor::MakeDirectory
	' This private method will create a directory on the server.
	' Expected Form Variables submitted from a device:
	'   destDir: the fully qualified file system path of the directory to create
	' Expected Behavior:
	'   The make directory method will create the directory specified in the form variables
	'   of the request stream.  
	' Expected Return:
	'   If successful, nothing.
	'   XRXBADNAME if the name is empty.
	'   XRXDIREXISTS if the directory already exists.
	'   XRXERROR if the directory cannot be created.
	' Script Exploit Opportunities:
	'   This function could be used to create enough directories to exceed the file system's
	'   limits.  This script does not prevent this.
	Private Sub MakeDirectory(arg_objUpload)
		' Our local variables.
		Dim objFSO, strDirectoryName, strReturn
		' Initialize our return state
		strReturn = ""
		' Get the name of the directory to create from the form variables in the request stream.
		strDirectoryName = arg_objUpload.Form.Item(att_constFormVarNameDestinationDirectory)
		' Make sure we have a meaningful directory name to create.  Please note that this function
		' allows a directory of any name to be created.
		If ("" <> strDirectoryName) And (att_constMaxPath >= Len(strDirectoryName)) Then
			' Setup an error handler just in case something goes wrong
			On Error Resume Next
			' Get us somebody to interact with the file system.
			Set objFSO = Server.CreateObject("Scripting.FileSystemObject")
			' Let's make sure the directory isn't already there.
			If NOT objFSO.FolderExists(strDirectoryName) Then
				' If it's not, then we should probably create it since that's what we were
				' asked to do.
				objFSO.CreateFolder(strDirectoryName)
				' Only flag the error if we have one
				If (0 <> Err.Number) Then
					' Flag the error
					strReturn = att_constError
					' Clear the error so that the rest of the code can proceed
					Err.Clear
				End If
			Else
				' Folder already exists, this is bad
				strReturn = att_constErrorDirExists
			End If
			' Make sure we allow the file system object to be garbage collected.
			Set objFSO = Nothing
		Else
			' The directory name is empty, that's bad
			strReturn = att_constErrorBadName
		End If
		' See if we should send an error back
		If ("" <> strReturn) Then
			Response.Write strReturn
		End If
		' vbsRequestExecutor::MakeDirectory is done.
	End Sub

	' Method: vbsRequestExecutor::RemoveDirectory
	' This private method will remove a directory on the server.  The directory must end in
	' lck, xsm  This minimizes the script from being exploited to delete unprotected areas of
	' the file system.
	' Expected Form Variables submitted from a device:
	'   destDir: the fully qualified file system path of the directory to remove.
	' Expected Behavior:
	'   The remove directory method will remove the directory specified in the form variables
	'   of the request stream.  
	' Expected Return:
	'   If successful, nothing
	'   XRXBADNAME if the requested directory isn't of the correct type or the name is empty.
	'   XRXNOTFOUND if the directory does not exist.
	'   XRXERROR if the directory cannot be deleted.
	' Script Exploit Opportunities:
	'   The remove directory operation could be used to remove any folder on the system provided
	'   that the running web process has sufficient rights to remove the folder.  Since this
	'   places the responsibility of insuring that the web process does not have rights to
	'   remove folders it shouldn't be onto the server configuration and that we cannot assume 
	'   that only our folders need to be removed on any given web server, we're going to limit 
	'   the extensions (type) of folders that this function will be able to delete.  This way, 
	'   the script cannot be used to remove any folder which the web process has rights to remove.
	Private Sub RemoveDirectory(arg_objUpload)
		' Our local variables.
		Dim objFSO, strDirectoryName, strReturn
		' Initialize our return state
		strReturn = ""
		' Get the name of the directory to remove from the form variables in the request stream.
		strDirectoryName = arg_objUpload.Form.Item(att_constFormVarNameDestinationDirectory)
		' Make sure that the directory ends in the agreed upon directory name ending convention.
		' This prevents this function from being used maliciously against the HTTP server.
		If CheckDirectoryExtension(strDirectoryName) Then
			' Setup an error handler just in case something goes wrong
			On Error Resume Next
			' Get us somebody to interact with the file system.
			Set objFSO = Server.CreateObject("Scripting.FileSystemObject")
			' Let's make sure the directory exists in the first place
			If objFSO.FolderExists(strDirectoryName) Then
				' Then remove the directory, deleting all files and subdirectories it might contain.
				objFSO.DeleteFolder(strDirectoryName)
				' Only flag the error if we have one
				If (0 <> Err.Number) Then
					' Flag the error
					strReturn = att_constError
					' Clear the error so that the rest of the code can proceed
					Err.Clear
				End If
			Else
				' Folder doesn't exists, this is bad
				strReturn = att_constErrorNotFound
			End If
			' Make sure we allow the file system object to be garbage collected.
			Set objFSO = Nothing
		Else
			' The directory name is empty or doesn't have the right extension, that's bad
			strReturn = att_constErrorBadName
		End If
		' See if we should send an error back
		If ("" <> strReturn) Then
			Response.Write strReturn
		End If
		' vbsRequestExecutor::RemoveDirectory is done.
	End Sub

	' Method: vbsRequestExecutor::DeleteDirectoryContents
	' This private method will remove a directory and all of its subdirectories and files 
	' on the server.  The directory must end in ".lck".  This minimizes the script from 
	' being exploited to delete unprotected areas of the file system.
	' Expected Form Variables submitted from a device:
	'   deleteDir: Zero to remove the directory also, One to keep the directory.
	'   destDir: the fully qualified file system path of the directory to remove
	' Expected Behavior:
	'   The delete directory contents method will delete the directory and all of its
	'   files and subdirectories specified in the form variables.
	'   of the request stream.  
	' Expected Return:
	'   If successful, nothing
	'   XRXBADNAME if the requested directory isn't of the correct type or the name is empty.
	'   XRXNOTFOUND if the directory does not exist.
	'   XRXERROR if the directory (or some of its files) cannot be deleted.
	' Not Supported:
	'   At present, the form variable "isRecursive" is not supported. The device team's test 
	'   HTML form contains this variable, but it is not used by the existing server-side script. 
	'   As such, it is not supported in this script.
	' Script Exploit Opportunities:
	'   The delete directory contents operation could be used to remove any folder and contained
	'   subfolders and files on the system provided that the running web process has sufficient 
	'   rights to remove the folder.  Since this places the responsibility of insuring that the 
	'   web process does not have rights to remove folders and delete files onto the server 
	'   configuration and that we cannot assume that only our folders need to be removed on any 
	'   given web server, we're going to limit the extensions (type) of folders that this function 
	'   will be able to delete.  This way, the script cannot be used to remove any folder which the 
	'   web process has rights to remove.
	Private Sub DeleteDirectoryContents(arg_objUpload)
		' Our local variables.
		Dim objFSO, strDirectoryName, bAlsoDeleteDirectory, strReturn
		' Initialize our return state
		strReturn = ""
		' Get the name of the directory in which to delete all of the files from the form 
		' variables in the request stream.
		strDirectoryName = arg_objUpload.Form.Item(att_constFormVarNameDestinationDirectory)
		' Get whether of not to also delete the directory from the form variables 
		' in the request stream.
		bAlsoDeleteDirectory = arg_objUpload.Form.Item(att_constFormVarNameDeleteDirectory)
		' Make sure that the directory ends in the agreed upon directory name ending convention.
		' This prevents this function from being used maliciously against the HTTP server.
		If CheckDirectoryExtension(strDirectoryName) Then
			' Get us somebody to interact with the file system.
			Set objFSO = Server.CreateObject("Scripting.FileSystemObject")
			' Let's make sure that the directory exists in the first place.
			If objFSO.FolderExists(strDirectoryName) Then
				' Setup an error handler just in case something goes wrong
				On Error Resume Next
				' Instead of enumerating and deleting the files in the directory,
				' just remove the directory as it will delete any files (and sub
				' directories contained therein.
				objFSO.DeleteFolder(strDirectoryName)
				' Only flag the error if we have one
				If (0 <> Err.Number) Then
					' Flag the error
					strReturn = att_constError
					' Clear the error so that the rest of the code can proceed
					Err.Clear
					' Since we're in error, make sure we don't try to create the directory.
					bAlsoDeleteDirectory = "1"
				End If
				' If we weren't asked to also delete the directory too
				If "0" = bAlsoDeleteDirectory Then
					' then recreate it.
					objFSO.CreateFolder(strDirectoryName)
					' Only flag the error if we have one
					If (0 <> Err.Number) Then
						' Flag the error
						strReturn = att_constError
						' Clear the error so that the rest of the code can proceed
						Err.Clear
					End If
				End If
			Else
				strReturn = att_constErrorNotFound
			End If
			' Make sure we allow the file system object to be garbage collected.
			Set objFSO = Nothing
		Else
			strReturn = att_constErrorBadName
		End If
		' See if we should send an error back
		If ("" <> strReturn) Then
			Response.Write strReturn
		End If
		' vbsRequestExecutor::DeleteDirectoryContents is done.
	End Sub

	' Method: vbsRequestExecutor::ListDirectory
	' This private method will list all of the files in a directory specified in 
	' the form variables in the request stream.  It will also lists any sub directories,
	' but not the files in the sub directories or their sub directories.
	' Expected Form Variables submitted from a device:
	'   destDir: the fully qualified file system path of the directory to list
	' Expected Behavior:
	'   The list directory method will enumerate the contents of the directory specified 
	'   in the form variables of the request stream.  
	' Expected Return:
	'   Nothing if no contents are found.  Otherwise it returns a line-feed (ASCII 10)
	'   delimited list of files and subdirectories.  Subdirectories are noted by appending
	'   the forward slash ("/") character to the end of the subdirectory's name.
	'   Otherwise: XRXBADNAME if the requested directory name is empty.
	'   XRXNOTFOUND if the directory does not exist.
	'   XRXERROR if the directory cannot be read.
	' Script Exploit Opportunities:
	'   This function could be used to determine the name of the files within a directory
	'   and use those names for malicious purposes like deleting files.  As this script
	'   limits activities like deleting files and dirctories, the only risk is that another
	'   script is used to perform the malicious activities. This script does not prevent that.
	Private Sub ListDirectory(arg_objUpload)
		' Our local variables.
		Dim objFSO, strDirectoryName, listFiles, strReturn
		' Initialize our return state
		strReturn = ""
		' Get the name of the directory to list from the form variables in the request stream.
		strDirectoryName = arg_objUpload.Form.Item(att_constFormVarNameDestinationDirectory)
		' Make sure we have a meaningful directory name to list.
		If ("" <> strDirectoryName) And (att_constMaxPath >= Len(strDirectoryName)) Then
			' Get us somebody to interact with the file system.
			Set objFSO = Server.CreateObject("Scripting.FileSystemObject")
			' Make sure the folder actually exists.
			If objFSO.FolderExists(strDirectoryName) Then
				' Setup an error handler just in case something goes wrong
				On Error Resume Next
				' Get us the list of entries found in the directory
				Set listFiles = objFSO.GetFolder(strDirectoryName)
				' Only flag the error if we have one
				If (0 <> Err.Number) Then
					' Flag the error
					strReturn = att_constError
					' Clear the error so that the rest of the code can proceed
					Err.Clear
					' Since we're in error, make sure we don't try to create the directory.
					bAlsoDeleteDirectory = "1"
				Else
					' For each of the file entries in the list
					For each filefound in listFiles.files
						' stream the name back in the response
						Response.Write filefound.Name
						' and follow it with a newline (or equivalent).
						Response.Write vblf
					Next
					' For each of the sub directory entries in the list
					For each filefound in listFiles.SubFolders
						' add a slash to the end of the name and stream it back in the response
						Response.Write filefound.Name + "/"
						' then follow it with a newline (or equivalent).
						Response.Write vblf
					Next
				End If
				' Make sure we allow the file system object to be garbage collected.
				Set objFSO = Nothing
			Else
				strReturn = att_constErrorNotFound
			End If
		Else
			strReturn = att_constErrorBadName
		End If
		' See if we should send an error back
		If ("" <> strReturn) Then
			Response.Write strReturn
		End If
		' vbsRequestExecutor::ListDirectory is done.
	End Sub

	' Method: vbsRequestExecutor::SaveFiles
	' This private method will save all of the files post as multi-part form data into 
	' the directory specified in the form variables of the request stream.
	' Expected Form Variables submitted from a device:
	'   destName: the name of the file to delete, without path information.
	'   destDir: the fully qualified file system path of the directory to list
	' Expected Behavior:
	'   The save file method will save the files found in the request to the directory
	'   specified in the request stream.
	'   Note: if accessing this page annonymously, the internet guest account must have
	'   write permission to the path.
	' Expected Return:
	'   If successful, nothing.
	'   XRXBADNAME if the file name is empty.
	'   XRXERROR if the file cannot be written.
	' Script Exploit Opportunities:
	'   The save files operation could be used to upload malicious scripts to the server.
	'   Once uploaded, if the server has execute permission of these scripts, a user can
	'   effectively, remotely invoke the malicious code. The easiest solution to prevent this
	'   would be to remove execute permissions to the files this function writes.  This however
	'   does not apply to Windows-based file systems.  The other alternative would be limit
	'   the types of files this function uploads.  However, doing so would greatly limit this
	'   scripts ability to handle new uploads.  As lack of extensibility is greater than the
	'   risk of the use of this function to upload malicious scripts, this function does not
	'   prevent the upload of potentially malicious scripts.
	Private Sub SaveFiles(arg_objUpload)
		' Our local variables.
		Dim lngLoop, strDestinationName, strPathName, strReturn
		' Initialize our return state
		strReturn = ""
		' Get the path to save to from the form variables in the request stream.
		strPathName = arg_objUpload.Form.Item(att_constFormVarNameDestinationDirectory)
		' Get the name of the directory to save the files in from the form variables
		' in the request stream.
		strDestinationName = arg_objUpload.Form.Item(att_constFormVarNameDestinationName)
		' Make sure we have a meaningful location to save to.
		If ("" <> strDestinationName) And ("" <> strPathName) And (att_constMaxPath >= Len(strDestinationName)) And (att_constMaxPath >= Len(strPathName)) Then
			' Setup an error handler just in case something goes wrong
			On Error Resume Next
			' Assuming there could be multiple files
			For lngLoop = 0 to arg_objUpload.Files.Count - 1
				' Set the name of the file
				arg_objUpload.Files.Item(lngLoop).FileName = strDestinationName
				' Tell the actual file object to save itself
				arg_objUpload.Files.Item(lngLoop).Save(strPathName)
				' Only flag the error if we have one
				If (0 <> Err.Number) Then
					' Flag the error
					strReturn = att_constError
					' Clear the error so that the rest of the code can proceed
					Err.Clear
					' Cancel our loop
					lngLoop =  arg_objUpload.Files.Count
				Else
					' Put our count out of range in case there are multiple files.  This prevents
					' us from overwrites and someone attempting to crash us by sending us a zillion files.
					lngLoop = arg_objUpload.Files.Count + 1
				End If
			Next
		Else
			att_constErrorBadName
		End If
		' See if we should send an error back
		If ("" <> strReturn) Then
			Response.Write strReturn
		End If
		' vbsRequestExecutor::SaveFiles is done.
	End Sub

	' Method: vbsRequestExecutor::CheckFileExtension
	' Internal Helper Only
	' To help with processing time of the script, the construction of the file name extensions
	' to check and the check itself have been encapsulated into this function.  This way, the
	' work is only done when the function is called.  Please note that the function is not
	' presently called more than once per any form post.  That enables the payoff of this 
	' strategy.
	' This function is essentially the same as vbsRequestExecutor::CheckDirectoryExtension
	' and could make use of a third function to remove the duplicated code.  At present, it
	' remains as is in the event that files and directories are determined to require differing
	' checks.
	Private Function  CheckFileExtension(arg_fileName)
		' Our local variables.
		Dim iCounter, objList, strTester, strExtension, arrItems

		' Initialize our returning value to false which signifies that we didn't find a match
		CheckFileExtension = False
		' Get us a collection to store the list of valid types
		Set objList = New vbsCollection

		' Make sure we have something to check
		If ("" <> arg_fileName) And (att_constMaxPath >= Len(arg_fileName)) Then
			' Setup up the list of extensions to test against
			objList.Add ".xst", ".XST"
			objList.Add ".dat", ".DAT"
			objList.Add ".txt", ".TXT"

			' See if any of the extensions match the requested file name
			for iCounter = 0 To objList.Count - 1
				' Get the extenstion part of the string
				strExtension = objList.Item(iCounter)
				strTester = Right(arg_fileName, Len(strExtension))
				' If the extensions match (use case insensitive comparison
				If StrComp(strExtension,  strTester, vbTextCompare) Then
					' then set our returning value to true which signifies we found a match
					CheckFileExtension = True
					iCounter = objList.Count + 1
				End If
			Next
		End If
		' vbsRequestExecutor::CheckFileExtension is done.
	End Function

	' Method: vbsRequestExecutor::CheckDirectoryExtension
	' Internal Helper Only
	' To help with processing time of the script, the construction of the directory name extensions
	' to check and the check itself have been encapsulated into this function.  This way, the
	' work is only done when the function is called.  Please note that the function is not
	' presently called more than once per any form post.  That enables the payoff of this 
	' strategy.
	' This function is essentially the same as vbsRequestExecutor::CheckFileExtension
	' and could make use of a third function to remove the duplicated code.  At present, it
	' remains as is in the event that files and directories are determined to require differing
	' checks.
	Private Function  CheckDirectoryExtension(arg_dirName)
		' Our local variables.
		Dim iCounter, objList, strTester, strExtension, arrItems

		' Initialize our returning value to false which signifies that we didn't find a match
		CheckDirectoryExtension = False
		' Get us a collection to store the list of valid types
		Set objList = New vbsCollection

		' Make sure we have something to check and that it doesn't exceed our bounds
		If ("" <> arg_dirName) And (att_constMaxPath >= Len(arg_dirName)) Then
			' Setup up the list of extensions to test against
			objList.Add ".lck", ".LCK"
			objList.Add ".xsm", ".XSM"

			' See if any of the extensions match the requested file name
			for iCounter = 0 To objList.Count - 1
				' Get the extenstion part of the string
				strExtension = objList.Item(iCounter)
				strTester = Right(arg_dirName, Len(strExtension))
				' If the extensions match (use case insensitive comparison
				If StrComp(strExtension,  strTester, vbTextCompare) Then
					' then set our returning value to true which signifies we found a match
					CheckDirectoryExtension = True
					iCounter = objList.Count + 1
				End If
			Next
		End If
		' vbsRequestExecutor::CheckDirectoryExtension is done.
	End Function

	' Method: vbsRequestExecutor::BuildFullyQualifiedPath
	' Internal Helper Only
	' This function ensures that there is a slash on the end of the path before appending
	' the file name onto it.
	Private Function  BuildFullyQualifiedPath(arg_dirName, arg_fileName)
		Dim strSlash
		' Initialize our returning value to an empty string in case we get garbage
		BuildFullyQualifiedPath = ""
		' Make sure we have a directory and a file name to work with
		If ("" <> arg_dirName) And (att_constMaxPath >= Len(arg_dirName)) And ("" <> arg_fileName) And (att_constMaxPath >= Len(arg_fileName)) Then
			' Be nice and add the slash in just in case it was omitted. 
			' (No need to worry about direction of the slash, just deal with a missing one.)
			strSlash = Right(strPathName, 1)
			If "/" = strSlash or "\" = strSlash Then
				BuildFullyQualifiedPath = arg_dirName + arg_fileName
			Else
				BuildFullyQualifiedPath = arg_dirName + "\" + arg_fileName
			End If
		End If
		' vbsRequestExecutor::BuildFullyQualifiedPath is done.
	End FUnction

' End of vbsRequestExecutor class.
End Class

' --------------------------------------------------------------------------------------------------'
' vbsUpload is a handy, commonly used class for handling file uploads.  It includes the 
' funcationality to process the incoming multi-part request and to save the file to a specified
' location.  It is included unmodified and as currently used, without the addition of any additional 
' comments for maintainability or feature enhancement.  Given the object oriented nature of this
' approach and the use of currently established naming conventions, it is openly readable at a 
' novice programmer level.
'
' Class hierarchy in Schlarr-Mellor notation is:
'   vbsUload -> vbsCollection ->> vbsFile
' 
'========================================================='
'	Class vbsUpload will parse the binary contents of the '
'	request, and populate the Form and Files collections. '
'	This works based on the RFC1867 
'	(http://www.ietf.org/rfc/rfc1867.txt?number=1867)	  '
'========================================================='
Class vbsUpload
	Private m_objFiles
	Private m_objForm

	Public Property Get Form()
		Set Form = m_objForm
	End Property

	Public Property Get Files()
		Set Files = m_objFiles
	End Property

	Private Sub Class_Initialize()
		Set m_objFiles = New vbsCollection
		Set m_objForm = New vbsCollection
		ParseRequest
	End Sub

	Private Sub ParseRequest()
		Dim lngTotalBytes, lngPosBeg, lngPosEnd, lngPosBoundary, lngPosTmp, lngPosFileName
		Dim strBRequest, strBBoundary, strBContent
		Dim strName, strFileName, strContentType, strValue, strTemp
		Dim objFile

		'Grab the entire contents of the Request as a Byte string
		lngTotalBytes = Request.TotalBytes
		strBRequest = Request.BinaryRead(lngTotalBytes)

		'Find the first Boundary
		lngPosBeg = 1
		lngPosEnd = InStrB(lngPosBeg, strBRequest, UStr2Bstr(Chr(13)))
		If lngPosEnd > 0 Then
			strBBoundary = MidB(strBRequest, lngPosBeg, lngPosEnd - lngPosBeg)
			lngPosBoundary = InStrB(1, strBRequest, strBBoundary)
		End If
		If strBBoundary = "" Then
		'The form must have been submitted *without* ENCTYPE="multipart/form-data"
		'But since we already called Request.BinaryRead, we can no longer access
		'the Request.Form collection, so we need to parse the request and populate
		'our own form collection.
			lngPosBeg = 1
			lngPosEnd = InStrB(lngPosBeg, strBRequest, UStr2BStr("&"))
			Do While lngPosBeg < LenB(strBRequest)
				'Parse the element and add it to the collection
				strTemp = BStr2UStr(MidB(strBRequest, lngPosBeg, lngPosEnd - lngPosBeg))
				lngPosTmp = InStr(1, strTemp, "=")
				strName = URLDecode(Left(strTemp, lngPosTmp - 1))
				strValue = URLDecode(Right(strTemp, Len(strTemp) - lngPosTmp))
				m_objForm.Add strName, strValue
				'Find the next element
				lngPosBeg = lngPosEnd + 1
				lngPosEnd = InStrB(lngPosBeg, strBRequest, UStr2BStr("&"))
				If lngPosEnd = 0 Then lngPosEnd = LenB(strBRequest) + 1
			Loop
		Else
		'The form was submitted with ENCTYPE="multipart/form-data"
		'This implies that the form data set is encoded so that each form field
		'(more exactly, each "control") is presented in a format suitable for that field,
		'and the data set as a whole is a 'multipart message containing 
		'those presentations as its components.
		'Loop through all the boundaries, and parse them into either the
		'Form or Files collections.
			Do Until (lngPosBoundary = InStrB(strBRequest, strBBoundary & UStr2Bstr("--")))
				'Get the element name
				lngPosTmp = InStrB(lngPosBoundary, strBRequest, UStr2BStr("Content-Disposition"))
				lngPosTmp = InStrB(lngPosTmp, strBRequest, UStr2BStr("name="))
				lngPosBeg = lngPosTmp + 6
				lngPosEnd = InStrB(lngPosBeg, strBRequest, UStr2BStr(Chr(34)))
				strName = BStr2UStr(MidB(strBRequest, lngPosBeg, lngPosEnd - lngPosBeg))
				'Look for an element named 'filename'
				lngPosFileName = InStrB(lngPosBoundary, strBRequest, UStr2BStr("filename="))
				'If found, we have a file, otherwise it is a normal form element
				If lngPosFileName <> 0 And lngPosFileName < InStrB(lngPosEnd, strBRequest, strBBoundary) Then 'It is a file
					'Get the FileName
					lngPosBeg = lngPosFileName + 10
					lngPosEnd = InStrB(lngPosBeg, strBRequest, UStr2BStr(chr(34)))
					strFileName = BStr2UStr(MidB(strBRequest, lngPosBeg, lngPosEnd - lngPosBeg))
					'Get the ContentType
					lngPosTmp = InStrB(lngPosEnd, strBRequest, UStr2BStr("Content-Type:"))
					lngPosBeg = lngPosTmp + 14
					lngPosEnd = InstrB(lngPosBeg, strBRequest, UStr2BStr(chr(13)))
					strContentType = BStr2UStr(MidB(strBRequest, lngPosBeg, lngPosEnd - lngPosBeg))
					'Get the Content
					lngPosBeg = lngPosEnd + 4
					lngPosEnd = InStrB(lngPosBeg, strBRequest, strBBoundary) - 2
					strBContent = MidB(strBRequest, lngPosBeg, lngPosEnd - lngPosBeg)
					If strFileName <> "" And strBContent <> "" Then
						'Create the File object, and add it to the Files collection
						Set objFile = New vbsFile
						objFile.Name = strName
						objFile.FileName = Right(strFileName, Len(strFileName) - InStrRev(strFileName, "\"))
						objFile.ContentType = strContentType
						objFile.Blob = strBContent
						m_objFiles.Add strName, objFile
					End If
				Else 'It is a form element
					'Get the value of the form element
					lngPosTmp = InStrB(lngPosTmp, strBRequest, UStr2BStr(chr(13)))
					lngPosBeg = lngPosTmp + 4
					lngPosEnd = InStrB(lngPosBeg, strBRequest, strBBoundary) - 2
					strValue = BStr2UStr(MidB(strBRequest, lngPosBeg, lngPosEnd - lngPosBeg))
					'Add the element to the collection
					m_objForm.Add strName, strValue
				End If
				'Move to Next Element
				lngPosBoundary = InStrB(lngPosBoundary + LenB(strBBoundary), strBRequest, strBBoundary)
			Loop
		End If
	End Sub

	Private Function BStr2UStr(BStr)
	'Byte string to Unicode string conversion
		Dim lngLoop
		BStr2UStr = ""
		For lngLoop = 1 to LenB(BStr)
			BStr2UStr = BStr2UStr & Chr(AscB(MidB(BStr,lngLoop,1))) 
		Next
	End Function

	Private Function UStr2Bstr(UStr)
	'Unicode string to Byte string conversion
		Dim lngLoop
		Dim strChar
		UStr2Bstr = ""
		For lngLoop = 1 to Len(UStr)
			strChar = Mid(UStr, lngLoop, 1)
			UStr2Bstr = UStr2Bstr & ChrB(AscB(strChar))
		Next
	End Function

	Private Function URLDecode(Expression)
		Dim strSource, strTemp, strResult
		Dim lngPos
		strSource = Replace(Expression, "+", " ")
		For lngPos = 1 To Len(strSource)
			strTemp = Mid(strSource, lngPos, 1)
			If strTemp = "%" Then
				If lngPos + 2 < Len(strSource) Then
					strResult = strResult & Chr(CInt("&H" & Mid(strSource, lngPos + 1, 2)))
					lngPos = lngPos + 2
				End If
			Else
				strResult = strResult & strTemp
			End If
		Next
		URLDecode = strResult
	End Function

End Class

' Helper class for vbsUpload.  Provided to de-complicate the vbsUpload class and for ease 
' of readability and code maintainability.
'========================================================='
'	Class vbsCollection is a pseudo-collection.			  '
'	It is not a real "collection".						  '
'========================================================='
Class vbsCollection
	Private m_objDicItems

	Private Sub Class_Initialize()
		Set m_objDicItems = Server.CreateObject("Scripting.Dictionary")
		m_objDicItems.CompareMode = vbTextCompare
	End Sub

	Public Property Get Count()
		Count = m_objDicItems.Count
	End Property

	Public Default Function Item(Index)
		Dim arrItems
		If IsNumeric(Index) Then
			arrItems = m_objDicItems.Items
			If IsObject(arrItems(Index)) Then
				Set Item = arrItems(Index)
			Else
				Item = arrItems(Index)
			End If
		Else
			If m_objDicItems.Exists(Index) Then
				If IsObject(m_objDicItems.Item(Index)) Then
					Set Item = m_objDicItems.Item(Index)
				Else
					Item = m_objDicItems.Item(Index)
				End If
			End If
		End If
	End Function

	Public Function Key(Index)
		Dim arrKeys
		If IsNumeric(Index) Then
			arrKeys = m_objDicItems.Keys
			Key = arrKeys(Index)
		End If
	End Function

	Public Sub Add(Name, Value)
		If m_objDicItems.Exists(Name) Then
			m_objDicItems.Item(Name) = Value
		Else
			m_objDicItems.Add Name, Value
		End If
	End Sub
End Class

' Helper class for vbsUpload.  Provided to de-complicate the vbsUpload class and for ease 
' of readability and code maintainability.
Class vbsFile
'========================================================='
'	Class vbsFile is used as a container for a file		  '
'	sent via an http multipart/form-data post.			  '
'========================================================='
	Private m_strName
	Private m_strContentType
	Private m_strFileName
	Private m_Blob

	Public Property Get Name() : Name = m_strName : End Property
	Public Property Let Name(vIn) : m_strName = vIn : End Property
	Public Property Get ContentType() : ContentType = m_strContentType : End Property
	Public Property Let ContentType(vIn) : m_strContentType = vIn : End Property
	Public Property Get FileName() : FileName = m_strFileName : End Property
	Public Property Let FileName(vIn) : m_strFileName = vIn : End Property
	Public Property Get Blob() : Blob = m_Blob : End Property
	Public Property Let Blob(vIn) : m_Blob = vIn : End Property

	Public Sub Save(Path)
		Dim objFSO, objFSOFile
		Dim lngLoop
		Set objFSO = Server.CreateObject("Scripting.FileSystemObject")
		Set objFSOFile = objFSO.CreateTextFile(objFSO.BuildPath(Path, m_strFileName))
		For lngLoop = 1 to LenB(m_Blob)
			objFSOFile.Write Chr(AscB(MidB(m_Blob, lngLoop, 1)))
		Next
		objFSOFile.Close
	End Sub
End Class

' End of the class library section of this code
' --------------------------------------------------------------------------------------------------'

' !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
' !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
' Start of the actual working part of this code.

' Get us some memory for the object that is going to do the work.
Dim objRequestExecutor
' Get us somebody who can handle the requests
Set objRequestExecutor = New vbsRequestExecutor
' and tell them to do it!
objRequestExecutor.Run()
' that's it!

' End of the actual working part of this code.
' !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
' !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
%>
